@@ -3,7 +3,7 @@ use bevy_ascii_terminal::*;
 use bevy_tiled_camera::*;
 
 fn main() {
-    App::build()
+    App::new()
         .add_plugins(DefaultPlugins)
         .add_plugin(TerminalPlugin)
         .add_plugin(TiledCameraPlugin)
@@ -14,11 +14,13 @@ fn main() {
 }
 
 fn spawn_terminal(mut commands: Commands) {
-    let size = (20, 3);
+    let size = [20, 3];
     let mut term_bundle = TerminalBundle::new().with_size(size);
 
     term_bundle.terminal.draw_border_single();
-    term_bundle.terminal.put_string((1, 1), "Press spacebar");
+    term_bundle
+        .terminal
+        .put_string_color([1, 1], "Press spacebar", BLUE, BLACK);
 
     commands.spawn_bundle(term_bundle);
 
@@ -35,7 +37,13 @@ fn hello_world(keys: Res<Input<KeyCode>>, mut q: Query<&mut Terminal>) {
         for mut term in q.iter_mut() {
             term.clear();
             term.draw_border_single();
-            term.put_string((1, 1), "Hello, world!");
+            term.put_char_color([1, 1], 'H', WHITE, GREY);
+            term.put_char_color([2, 1], 'e', BLUE, WHITE);
+            term.put_char_color([3, 1], 'l', GREEN, BLUE);
+            term.put_char_color([4, 1], 'l', RED, GREEN);
+            term.put_char_color([5, 1], 'o', GREEN, GREY);
+
+            term.put_string([6, 1], " World!");
         }
     }
 }
