@@ -4,7 +4,7 @@ use bevy_ascii_terminal::{TerminalBundle, TerminalPlugin};
 use bevy_tiled_camera::*;
 
 fn main() {
-    App::build()
+    App::new()
         .add_plugins(DefaultPlugins)
         .add_plugin(TerminalPlugin)
         .add_plugin(TiledCameraPlugin)
@@ -16,15 +16,15 @@ fn main() {
 fn make_terminal(
     commands: &mut Commands,
     pos: Vec3,
-    size: (u32, u32),
-    pivot: (f32, f32),
+    size: [u32; 2],
+    pivot: [f32; 2],
     string: &str,
 ) {
     let mut term_bundle = TerminalBundle::new().with_size(size);
     term_bundle.renderer.terminal_pivot.0 = Vec2::from(pivot);
 
     term_bundle.terminal.draw_border_single();
-    term_bundle.terminal.put_string((1, 1), string);
+    term_bundle.terminal.put_string([1, 1], string);
 
     term_bundle.transform.translation = pos;
 
@@ -32,12 +32,12 @@ fn make_terminal(
 }
 
 fn setup(mut commands: Commands) {
-    let term_size = (15, 3);
-    let view_size = (40, 21);
+    let term_size = [15, 3];
+    let view_size = [40, 21];
 
     let origin = Vec3::ZERO;
-    let right = Vec3::X * view_size.0 as f32;
-    let up = Vec3::Y * view_size.1 as f32;
+    let right = Vec3::X * view_size[0] as f32;
+    let up = Vec3::Y * view_size[1] as f32;
 
     let pivot_bl = Vec2::ZERO;
     let pivot_right = Vec2::X;
@@ -81,6 +81,6 @@ fn setup(mut commands: Commands) {
         TiledCameraBundle::new()
             .with_pixels_per_tile(8)
             .with_tile_count(view_size)
-            .with_camera_position(view_pos.as_f32().into()),
+            .with_camera_position(view_pos.as_vec2().into()),
     );
 }
