@@ -7,7 +7,7 @@ use bevy_ascii_terminal::{renderer::TileScaling, TerminalBundle, TerminalPlugin}
 use bevy_tiled_camera::*;
 
 fn main() {
-    App::build()
+    App::new()
         .init_resource::<FontIndex>()
         .add_plugins(DefaultPlugins)
         .add_plugin(TerminalPlugin)
@@ -22,11 +22,11 @@ fn main() {
 struct FontIndex(pub usize);
 
 fn setup(mut commands: Commands, mut meshes: ResMut<Assets<Mesh>>) {
-    let size = (20, 3);
+    let size = [20, 3];
     let mut term_bundle = TerminalBundle::new().with_size(size);
 
     term_bundle.terminal.draw_border_single();
-    term_bundle.terminal.put_string((1, 1), "Press spacebar");
+    term_bundle.terminal.put_string([1, 1], "Press spacebar");
     commands.spawn_bundle(term_bundle);
 
     commands.spawn_bundle(PbrBundle {
