@@ -8,7 +8,7 @@ use rand::prelude::ThreadRng;
 use rand::Rng;
 
 fn main() {
-    App::build()
+    App::new()
         .init_resource::<Pause>()
         .add_plugins(DefaultPlugins)
         .add_plugin(TerminalPlugin)
@@ -21,14 +21,14 @@ fn main() {
         .run();
 }
 
-#[derive(Default)]
+#[derive(Component, Default)]
 struct Pause(bool);
 
 fn setup(mut commands: Commands) {
-    let size = (80, 50);
+    let size = [80, 50];
 
     let term = TerminalBundle::new().with_size(size);
-    commands.spawn_bundle(term).insert(Pause);
+    commands.spawn_bundle(term).insert(Pause::default());
 
     commands.spawn_bundle(
         TiledCameraBundle::new()
@@ -56,7 +56,8 @@ fn spam_terminal(keys: Res<Input<KeyCode>>, mut pause: ResMut<Pause>, mut q: Que
     let mut rng = rand::thread_rng();
     for mut term in q.iter_mut() {
         for t in term.iter_mut() {
-            let glyph = (rng.gen_range(0, 255) as u8) as char;
+            let index = rng.gen_range(0, 255) as u8;
+            let glyph = code_page_437::index_to_glyph(index);
             let fg = rand_color(&mut rng);
             let bg = rand_color(&mut rng);
 
@@ -66,8 +67,8 @@ fn spam_terminal(keys: Res<Input<KeyCode>>, mut pause: ResMut<Pause>, mut q: Que
                 bg_color: bg,
             }
         }
-        term.clear_box((0, 0), (25, 3));
+        term.clear_box([0, 0], [25, 3]);
         term.draw_border_single_color(WHITE, BLACK);
-        term.put_string((1, 1), "Press space to pause");
+        term.put_string([1, 1], "Press space to pause");
     }
 }
