@@ -121,7 +121,12 @@ impl From<[f32; 4]> for TileColor {
 
 impl From<TileColor> for [f32; 4] {
     fn from(c: TileColor) -> Self {
-        [c.r as f32, c.g as f32, c.b as f32, c.a as f32]
+        [
+            c.r as f32 / 255.0,
+            c.g as f32 / 255.0,
+            c.b as f32 / 255.0,
+            c.a as f32 / 255.0,
+        ]
     }
 }
 
