@@ -21,13 +21,13 @@
 //! use bevy_tiled_camera::*;
 //!
 //! fn setup(mut commands: Commands) {
-//!     let size = (20, 3);
+//!     let size = [20, 3];
 //!
 //!     let mut term_bundle = TerminalBundle::new().with_size(size);
 //!     let terminal = &mut term_bundle.terminal;
 //!
 //!     terminal.draw_border_single();
-//!     terminal.put_string((1, 1), "Hello world!");
+//!     terminal.put_string([1, 1], "Hello world!");
 //!
 //!     commands.spawn_bundle(term_bundle);
 //!
@@ -36,7 +36,7 @@
 //! }
 //!
 //! fn main () {
-//!     App::build()
+//!     App::new()
 //!     .add_plugins(DefaultPlugins)
 //!     .add_plugin(TerminalPlugin)
 //!     .add_plugin(TiledCameraPlugin)
@@ -54,27 +54,15 @@ mod terminal;
 pub use color::*;
 pub use terminal::{BorderGlyphs, Terminal, Tile};
 
-use bevy::prelude::*;
-use renderer::{TerminalRendererBundle, TerminalRendererPlugin};
+pub use renderer::code_page_437;
+pub use renderer::material::BuiltInFontHandles;
+pub use renderer::material::TerminalMaterial;
 
-pub use renderer::TerminalAssetLoadState;
+use bevy::prelude::*;
 
 /// A bundle with all the required components for a terminal.
 ///
-/// Can specify properties of the terminal on initilaization.
-///
-/// # Example
-/// ```
-/// use bevy_ascii_terminal::*;
-/// use bevy::prelude::*;
-///
-/// fn setup(mut commands: Commands) {
-///     commands.spawn_bundle(TerminalBundle::new()
-///         .with_size((10,10))
-///         .with_font("jt_curses_12x12.png"));
-///     }
-///
-/// ```
+/// Can specify some properties of the terminal on initilaization.
 #[derive(Bundle, Default)]
 pub struct TerminalBundle {
     pub terminal: Terminal,
@@ -82,7 +70,7 @@ pub struct TerminalBundle {
     pub global_transform: GlobalTransform,
 
     #[bundle]
-    pub renderer: TerminalRendererBundle,
+    pub renderer: renderer::TerminalRendererBundle,
 }
 
 impl TerminalBundle {
@@ -91,32 +79,16 @@ impl TerminalBundle {
     }
 
     /// Set the initial size of the terminal.
-    pub fn with_size(mut self, size: (u32, u32)) -> Self {
+    pub fn with_size(mut self, size: [u32; 2]) -> Self {
         self.terminal.resize(size);
         self
     }
-
-    /// Set the initial font for the terminal. This should be the file name of the font
-    /// in the `assets/textures` directory, including the extension.
-    ///
-    /// # Example
-    ///
-    /// ```
-    /// use bevy_ascii_terminal::*;
-    ///
-    /// let bundle = TerminalBundle::new()
-    ///     .with_font("zx_evolution_8x8.png");
-    /// ```
-    pub fn with_font(mut self, font_name: &str) -> Self {
-        self.renderer.font.change_font(font_name);
-        self
-    }
 }
 
 /// Plugin for terminal rendering and related components and systems.
 pub struct TerminalPlugin;
 impl Plugin for TerminalPlugin {
-    fn build(&self, app: &mut AppBuilder) {
-        app.add_plugin(TerminalRendererPlugin);
+    fn build(&self, app: &mut App) {
+        app.add_plugin(renderer::TerminalRendererPlugin);
     }
 }
